﻿<h1>Frequently-asked Questions</h1>

<div id="index">

	<h2>Troubleshooting Error Messages</h2>

	<ul>
		<li><strong>Could not modify stub.dll</strong></li>
		<li><strong>Could not load .Net interface library bridge__.dll (Error 0x0000007e)</strong> when attempting to use Microsoft.Net classes from Dyalog</li>
		<li><strong>File or assembly name bridgennn, or one of its dependencies, was not found</strong> when trying to use a Dyalog-based .Net assembly from another .Net environment</li>
		<li><strong>Error IA8</strong> during installation</li>
	</ul>


	<h2>Installation and Configuration</h2>

	<ul>
		<li>How do I specify MAXWS for bound Executables?</li>
		<li>Why can't I set MAXWS to use all the memory in my 32-bit system?</li>
		<li>Is Dyalog compatible with Windows Vista?</li>
	</ul>

</div><!-- index -->

<h2>Error message: Could not modify stub.dll</h2>

<p>
	The error message <q>Could not modify stub dll (Error: failed to load runtime
	engine)</q> means that Dyalog APL is unable to find the file named
	<code>dyalognnn[rt].dll</code> (where <code>nnn</code> is a set of digits
	which will vary according to the Dyalog version). This file needs to be on the
	system path (names in the <code>PATH</code> environment variable).
</p>

<p>
	Under Windows XP, you can check the definition of <code>PATH</code> and other
	environment variables by going to
	<code>Control Panel / System / Advanced / Environment Variables</code>.
</p>

<h3>Version 10.1</h3>

<p>
	The installer should create an environment variable named
	<code>DYALOGNETDIR101</code> and add <code>%DYALOGNETDIR101%;</code> to the
	path. Verify that <code>DYALOGNETDIR101</code> points to the Dyalog
	<code>dotnet</code> folder and that <code>%DYALOGNETDIR101%</code> is listed
	in the <code>PATH</code>.
</p>

<h3>Version 11.0:</h3>

<p>
	The version 11.0 installation adds a folder to the <code>PATH</code> without
	using an intermediate environment variable. Verify that the Dyalog
	<code>bin</code> folder is in your system <code>PATH</code>.
</p>

<p>
	By default, the name of this folder is
	<code>C:\Program Files\Dyalog\Dyalog APL 11.0\bin\</code>
</p>


<h2>Error message: Could not load .Net interface library bridge__.dll. (Error 0x0000007e)</h2>

<h3>For Version 11.0</h3>

<p>
	The default 11.0 path is
	<code>C:\Program Files\Dyalog\Dyalog APL 11.0\bin\</code>
</p>

<p>
	If you installed to the default directory this path should be in your system
	path. Verify this by checking the <code>PATH</code> variable in
	<code>Control Panel / System / Advanced / Environment Variables / System Variables</code>.
</p>

<p>
	Verify that the file <code>bridge110.dll</code> is in the above directory
	<em>and</em> in the Global Assembly Cache (<code>\windows\assembly</code>).
</p>

<h3>For Version 10.1</h3>

<p>
	Please check <code>PATH</code> and <code>DYALOGNETDIR101</code> variables in
	<code>Control Panel / System / Advanced / Environment Variables / System Variables</code>.
</p>

<p>
	<code>DYALOGNETDIR101</code> should be <code>C:\Dyalog101\dotnet</code> if
	<code>c:\dyalog101</code> is where you installed the APL interpreter.
</p>

<p>
	Path should have <code>%DYALOGNETDIR101%;</code> in it.
</p>

<p>
	For example, your Path might look like this:
</p>


<pre>%DYALOGNETDIR101%;
%DYALOGNETDIR%;
.;
%SystemRoot%\system32;
%SystemRoot%;
%SystemRoot%\System32\Wbem</pre>

<p>
	Also, verify that the file <code>bridge11.dll</code> is in the above directory
	<em>and</em> in the Global Assembly Cache.
</p>


<h2>Error message: File or assembly name bridgennn, or one of its dependencies, was not found.</h2>

<h3>Version 10.1</h3>

<p>
	If you move a Dyalog-based assembly to a different machine from the one on
	which it was created, and the Dyalog development environment is not installed
	on that machine, there are 3 files that need to be installed on the target
	machine:
</p>

<ol>
	<li>
		dyalognet.dll needs to be installed into the Global Assembly Cache
		(<code>c:\windows\assembly</code>)
	</li>
	<li>
		<code>bridge101.dll</code> needs to be installed into the GAC and must be on
		the system <code>PATH</code>
	</li>
	<li>
		<code>dyalog101rt.dll</code> needs to be installed on the system <code>PATH</code>,
		or <code>dyalog101.dll</code> if the assembly is not exported as a runtime assembly
	</li>
</ol>

<p>
	You should also make sure you have the latest versions of these DLLs, which
	can be obtained from the <a href="http://www.dyalog.dk/dss/dss.htm">Dyalog DSS
	website</a>.
</p>

<h3>Version 11.0</h3>

<p>
	As above, except that the digits at the end of file names should be 110 rather
	than 101.
</p>


<h2>Error IA8</h2>

<p>
	This error occurs during the setup of the APL samples as IIS Virtual
	Directories. The setup program is trying to add them to the Default Web Site -
	on your computer, there is obviously no such item. You will have to set these
	up manually as described in Chapter 5 of the .NET Interface Guide.
</p>


<h2>How do I specify MAXWS for bound Executables?</h2>

<p>
	You can specify the <code>MAXWS</code> parameter by putting
	<code>maxws=32000</code> in the command line field of the Export dialogue.
</p>


<h2>Why can&rsquo;t I set MAXWS to use all the memory in my 32-bit system?</h2>

<p>
	Under Windows, each and every process is given an address space of 2Gb. This
	is not the same as the amount of memory it will occupy, but you can think of
	it as the maximum amount of memory it could use. The distinction between
	address space and memory is an important one. Address space is mapped to
	memory (physical memory plus page space) by Windows. While address space is
	contiguous (0 to 2Gb), the memory it is mapped to is not.
</p>

<p>
	The workspace associated with any Dyalog application has to be allocated a
	contiguous address space. The size of workspace that you can obtain is
	therefore ultimately restricted to the size of the largest area of contiguous
	address space in the process.
</p>

<p>
	Windows processes typically consist of a main program plus any number of DLLs
	(and their data) that get loaded dynamically on demand. DLLs do not
	necessarily get loaded neatly into a process at progressively incrementing
	addresses, instead they get loaded in a way that tends to fragment the address
	space into a series of chunks. Fragmentation of the address space will limit
	the maximum workspace available.
</p>

<p>
	Note that a Dyalog mapped file also requires contiguous address space, and the
	size of file you can map is limited to the maximum contiguous area of address
	space that is currently unallocated within the process.
</p>

<p>
	A <code>dyalog.exe</code> or <code>dyalogrt.exe</code> process is entirely
	controlled by Dyalog APL. When the process starts, it reserves an amount of
	contiguous address space for the workspace which is defined by the
	<code>MAXWS</code> parameter. If there is insufficient contiguous address
	space available in the process. APL will issue an appropriate error message
	and terminate. In practice, fragmentation of the address space by Windows DLLs
	reduces the maximum workspace available to Dyalog APL to around 1.6Gb.
</p>

<p>
	To get a larger contiguous address space for a larger <code>MAXWS</code>, you
	can use the following tools:
</p>

<ul>
	<li>
		<a href="http://www.sysinternals.com/Utilities/ListDlls.html">
			http://www.sysinternals.com/Utilities/ListDlls.html</a>
	</li>
	<li>
		<a href="http://msdn.microsoft.com/library/default.asp?url=/library/en-us/tools/tools/rebase.asp">
		http://msdn.microsoft.com/library/default.asp?url=/library/en-us/tools/tools/rebase.asp</a>
	</li>
</ul>

<h3>Some hints on rebasing DLLs:</h3>

<ol>
	<li>Make a safe backup copy of the DLL first.</li>
	<li>
		You cannot rebase a DLL while it is in use &mdash; for example, close
		<code>dyalog.exe</code> if you wish to rebase <code>dyares32.dll</code>.
	</li>
	<li>
		Various sources seem to indicate that the "best" range to rebase DLLs is
		0x60000000&mdash;0x68000000.
	</li>
</ol>

<h3>If the badly placed DLL is a system or device DLL</h3>

<p>you may need to do this:</p>

<ol>

	<li>
		Start Windows in Safe Mode with Command Prompt Enter commands:
<pre>
cd \tmp
copy c:\windows\system32\xpsp2res.dll c:\tmp
rebase -b 0x67000000 xpsp2res.dll
move c:\windows\system32\xpsp2res.dll c:\
copy xpsp2res.dll c:\windows\system32
</pre>
	</li>

	<li>Restart Windows.</li>
</ol>


<h2>Is Dyalog compatible with Windows Vista?</h2>

<p>
	There are a handful of known issues with running Dyalog under Windows Vista.
	We expect to solve most or all of them in January 2007.
</p>

<h3>Installation</h3>

<p>
	If you are running User Access Control, your user will need to be a Power User
	or Administrator in order to install Dyalog. See this
	<a href="http://msdn2.microsoft.com/en-us/library/aa480152.aspx#appcomp_topic4">
	MSDN article on User Account Control</a> for details.
</p>

<h3>APL Fonts</h3>

<p>
	Windows Vista fails to display some of the APL symbols if you use the standard 
	fonts shipped with Dyalog. The Dyalog Symbol font <a class="ttf" 
	href="http://www.dyalog.dk/downloads/dysymb.ttf">dysymb.ttf</a> can be used 
	instead. This font looks best with ClearType enabled. We are looking at 
	improving the display of the font at small sizes.
</p>

<h3>Help Files</h3>

<p>
	Windows Vista does not support the 32-bit .HLP file format used for Dyalog on-
	line help files. A set of new files in .CHM files will be available in
	January, together with a DSS patch to version 11.0 which will allow the menus
	and F1 help to work with the new files.
</p>